@@ -1,7 +1,9 @@
 {-# OPTIONS_GHC -Wno-simplifiable-class-constraints #-}
 
 {-# LANGUAGE AllowAmbiguousTypes #-}
+{-# LANGUAGE ConstraintKinds #-}
 {-# LANGUAGE DataKinds #-}
+{-# LANGUAGE EmptyCase #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
@@ -92,6 +94,7 @@ gconName = conIdToString . conId
 
 -- | The fixity of the first constructor.
 --
+-- >>> import GHC.Generics ((:*:)(..))
 -- >>> gconFixity (Just 0)
 -- Prefix
 -- >>> gconFixity ([] :*: id)
@@ -141,22 +144,22 @@ conIdToInt (ConId i) = i
 conIdToString :: forall a. Constructors a => ConId a -> String
 conIdToString = gConIdToString . fromConId
 
--- | All constructor identifiers. This must not be called on an empty type.
+-- | All constructor identifiers.
 --
 -- @
 -- 'gconNum' \@a = length ('conIdEnum' \@a)
 -- @
 conIdEnum :: forall a. Constructors a => [ConId a]
-conIdEnum = fmap ConId [0 .. n]
+conIdEnum = fmap ConId [0 .. n-1]
   where
-    ConId n = conIdMax @a
+    n = gConNum @(Rep a)
 
--- | This must not be called on an empty type.
-conIdMin :: forall a. Constructors a => ConId a
+-- | The first constructor. This must not be called on an empty type.
+conIdMin :: forall a. (Constructors a, NonEmptyType "conIdMin" a) => ConId a
 conIdMin = ConId 0
 
--- | This must not be called on an empty type.
-conIdMax :: forall a. Constructors a => ConId a
+-- | The last constructor. This must not be called on an empty type.
+conIdMax :: forall a. (Constructors a, NonEmptyType "conIdMax" a) => ConId a
 conIdMax = toConId gConIdMax
 
 -- | Get a 'ConId' by name.
@@ -219,9 +222,9 @@ instance (GConstructors f, GConstructors g) => GConstructors (f :+: g) where
     if i < nf then
       gConIdToString @f (GConId i)
     else
-      gConIdToString @g (GConId (i - nf - 1))
+      gConIdToString @g (GConId (i - nf))
     where
-      GConId nf = gConIdMax @f
+      nf = gConNum @f
   gConId (L1 x) = reGConId (gConId x)
   gConId (R1 y) = let GConId i = gConId y in GConId (nf + 1 + i)
     where
@@ -239,6 +242,13 @@ instance Constructor c => GConstructors (M1 C c f) where
   gConFixity = conFixity
   gConIsRecord = conIsRecord
 
+instance GConstructors V1 where
+  gConIdToString x = x `seq` error "gConIdToString: empty type"  -- Input should be empty.
+  gConId v = case v of {}
+  gConNum = 0
+  gConFixity v = case v of {}
+  gConIsRecord v = case v of {}
+
 -- *** Find a constructor tag by name
 
 type ConIdNamed' n t = GConIdNamedIf n t (GConIdNamed n (Rep t))
@@ -258,6 +268,60 @@ type family GConIdNamedIf (n :: Symbol) (t :: *) (o :: Maybe Nat) :: Nat where
     ('Text "No constructor named " ':<>: 'ShowType n
     ':<>: 'Text " in generic type " ':<>: 'ShowType t)
 
+-- *** Check that a type is not empty
+
+-- | Constraint that a generic type @a@ is not empty.
+-- Producing an error message otherwise.
+--
+-- The 'Symbol' parameter 'fname' is used only for error messages.
+--
+-- It is implied by the simpler constraint @'IsEmptyType' a ~ 'False@
+class    NonEmptyType_ fname a => NonEmptyType fname a
+instance NonEmptyType_ fname a => NonEmptyType fname a
+
+-- | Internal definition of 'NonEmptyType'.
+-- It is implied by the simpler constraint @'IsEmptyType' a ~ 'False@.
+--
+-- >>> :set -XTypeFamilies
+-- >>> :{
+-- conIdMin' :: (Constructors a, IsEmptyType a ~ 'False) => ConId a
+-- conIdMin' = conIdMin
+-- :}
+--
+-- >>> :{
+-- conIdMax' :: (Constructors a, IsEmptyType a ~ 'False) => ConId a
+-- conIdMax' = conIdMax
+-- :}
+type NonEmptyType_ fname a = (ErrorIfEmpty fname a (IsEmptyType a) ~ '())
+
+-- 'True' if the generic representation is @M1 D _ V1@.
+type family GIsEmptyType (r :: k -> *) :: Bool where
+  GIsEmptyType (M1 D _d V1) = 'True
+  GIsEmptyType (M1 D _d (M1 C _c _f)) = 'False
+  GIsEmptyType (M1 D _d (_f :+: _g)) = 'False
+
+-- | 'True' if the generic type @a@ is empty.
+type IsEmptyType a = IsEmptyType_ a
+
+-- | Internal definition of 'IsEmptyType'.
+type IsEmptyType_ a = GIsEmptyType (Rep a)
+
+-- | Throw an error if the boolean @b@ is true, meaning that the type @a@ is empty.
+--
+-- Example:
+--
+-- > ghci> data E deriving Generic
+-- > ghci> conIdMin :: ConId E
+--
+-- Error message:
+--
+-- > The function 'conIdMin' cannot be used with the empty type E
+type family ErrorIfEmpty (fname :: Symbol) (a :: *) (b :: Bool) :: () where
+  ErrorIfEmpty fname a 'True = TypeError
+    ('Text "The function '" ':<>: 'Text fname
+    ':<>: 'Text "' cannot be used with the empty type " ':<>: 'ShowType a)
+  ErrorIfEmpty fname a 'False = '()
+
 -- * Type families
 
 -- | 'Meta' field of the 'M1' type constructor.
